@@ -33,7 +33,7 @@
 # OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 # ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
-# $Id: Fast.pm,v 1.4 2005/11/15 00:55:42 hardaker Exp $
+# $Id: Fast.pm,v 1.8 2006/04/20 00:32:33 hardaker Exp $
 #
 package Net::DNS::ZoneFile::Fast;
 # documentation at the __END__ of the file
@@ -43,9 +43,10 @@ use 5.005;
 use vars qw($VERSION);
 use IO::File;
 use Net::DNS;
+use Net::DNS::RR;
 use MIME::Base64;
 
-$VERSION = '0.6';
+$VERSION = '0.7';
 
 my $MAXIMUM_TTL = 0x7fffffff;
 
@@ -74,6 +75,15 @@ my $fh;
 my @fhs;
 my @lns;
 
+# boot strap optional DNSSEC module functcions
+# (not optional if trying to parse a signed zone, but we don't need
+# these modules unless we are.
+eval {
+    require Net::DNS::RR::NSEC;
+    require Net::DNS::RR::DNSKEY;
+};
+
+
 sub parse
   {
       my %param;
@@ -169,7 +179,9 @@ sub parse
 	      $z->{mbox} = uc $z->{mbox} if defined $z->{mbox};
 	      $z->{txtdname} = uc $z->{txtdname} if defined $z->{txtdname};
 	  }
-	  my $newrec = Net::DNS::RR->new_from_hash(%$z);
+	  my $newrec = 
+	    Net::DNS::RR->new_from_hash(%$z);
+
 	  if ($newrec->{'type'} eq 'DNSKEY') {
 	      $newrec->setkeytag;
 	  }
@@ -622,7 +634,7 @@ sub parse_line
 	      $parse = \&parse_rrsig;
 	  } elsif (/\G(\d+)\s+(\d+)\s+($pat_maybefullname)\s+([^=]+=)\s*/gc) {
 	      # single-line
-	      $rrsig->{'siginceptation'} = $1;
+	      $rrsig->{'siginception'} = $1;
 	      $rrsig->{'keytag'} = $2;
 	      $rrsig->{'signame'} = $3;
 	      $rrsig->{'sig'} = $4;
@@ -700,7 +712,7 @@ sub parse_line
 		 nxtdname  => lc($1),
 		 typelist  => $2,
 		 typebm    =>
-		 Net::DNS::RR::NSEC::_typestr2typebm(split(/\s+/,$2)),
+		 Net::DNS::RR::NSEC::_typearray2typebm(split(/\s+/,$2)),
 		};
 	  } else {
 	      error("bad NSEC data");
@@ -773,17 +785,21 @@ sub parse_soa_name
 	      $soa->{$soa->{nextkey}} = $1;
 	  } elsif (/\G$pat_skip$/gc) {
 	      return;
-	  } elsif (/\G[ \t]*($pat_name\.)[ \t]/igc) {
+	  } elsif (/\G[ \t]*(\@)[ \t]/igc) {
+	      $soa->{$soa->{nextkey}} = $origin;
+	  }  elsif (/\G[ \t]*($pat_name\.)[ \t]/igc) {
 	      $soa->{$soa->{nextkey}} = $1;
 	  } else {
 	      error("expected valid $soa->{nextkey}");
 	  }
       } else {
-	  if (/\G[ \t]+($pat_maybefullname)/igc) {
+	  if (/\G[ \t]*($pat_maybefullname)/igc) {
 	      $soa->{$soa->{nextkey}} = $1;
 	  } elsif (/\G[ \t]*\($pat_skip$/igc) {
 	      $soa->{breakable} = 1;
 	      return;
+	  } elsif (/\G[ \t]*(\@)[ \t]/igc) {
+	      $soa->{$soa->{nextkey}} = $origin;
 	  } elsif (/\G[ \t]*\(/igc) {
 	      $soa->{breakable} = 1;
 	      $parse->();
@@ -821,7 +837,7 @@ sub parse_rrsig
       if ($rrsig->{'first'}) {
 	  delete $rrsig->{'first'};
 	  if (/\G\s*(\d+)\s+(\d+)\s+($pat_maybefullname)/gc) {
-	      $rrsig->{'siginceptation'} = $1;
+	      $rrsig->{'siginception'} = $1;
 	      $rrsig->{'keytag'} = $2;
 	      $rrsig->{'signame'} = $3;
 	  } else {
